import numpy as np
from candlelite import exception

__all__ = ['predict_bar']


# 估计K线的时间粒度
def predict_bar(candle: np.array, MINUTE_BAR_INTERVAL=60000) -> str:
    '''
    :param candle: 历史K线
    :param MINUTE_BAR_INTERVAL: 每分钟的时间间隔，默认单位毫秒

    支持的时间粒度估计: 秒s 分钟m 小时h 天d
    '''
    bar_interval = np.min(np.diff(candle[:, 0]))
    bar_int = bar_interval / MINUTE_BAR_INTERVAL
    if bar_int < 1:
        bar_int = bar_interval / (MINUTE_BAR_INTERVAL / 60)
        suffix = 's'
    elif bar_int <= 59:
        suffix = 'm'
        bar_int = bar_int
    elif bar_int <= 60 * 23:
        suffix = 'h'
        bar_int = bar_int / 60
    else:
        bar_int = bar_int / (60 * 24)
        suffix = 'd'

    if not bar_int == int(bar_int):
        raise exception.ExecuteException(
            func='predict_bar',
            msg='The predict bar result is {bar_int}{suffix}'.format(
                bar_int=str(bar_int),
                suffix=suffix,
            )
        )
    return '{bar_int}{suffix}'.format(bar_int=int(bar_int), suffix=suffix)


if __name__ == '__main__':
    pass



import sys


def cmd():
    msg_fmt = '{cmd:<30}{note}'
    command = sys.argv[1]
    if command == '--help':
        msgs = [
            msg_fmt.format(cmd='candlelite show_settings', note='Current settings file information'),
            msg_fmt.format(cmd='candlelite console_settings', note='Modify the settings file in the terminal'),
            msg_fmt.format(cmd='candlelite settings_path', note='Get settings file path'),
        ]
        print('\n'.join(msgs))
    elif command == 'show_settings':
        from candlelite.settings import show_settings
        show_settings()
    elif command == 'console_settings':
        from candlelite.settings import console_settings
        console_settings()
    elif command == 'settings_path':
        from candlelite.settings import get_settings_filepath
        print(get_settings_filepath())
    else:
        pmt = 'Error command. You can input [candlelite --help] to view the supported commands'
        print(pmt)



class AbstractEXP(Exception):
    error_msg: str

    def __str__(self):
        return self.error_msg

def get_error_msg(func,reason):
    msg = '[ERROR] [{func}] {reason}'.format(
        func= func,reason=reason
    )
    return msg


from candlelite.io import load, path, save
from typing import Union, Literal
import datetime
import numpy as np
from paux.param import to_local


class IO():
    CANDLE_DATE_BASE_DIR: str
    CANDLE_FILE_BASE_DIR: str
    TIMEZONE: str
    BAR: str

    # 获取candle具备数据的日期序列
    def get_candle_dates(
            self,
            instType: str,
            symbol: str,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.get_candle_dates(**to_local(locals()))

    # 获取全部的产品名称
    def get_symbols_all(
            self,
            instType: str,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.get_symbols_all(**to_local(locals()))

    # 加载一个产品已有的全部K线
    def load_candle_all(
            self,
            instType: str,
            symbol: str,
            base_dir: str,
            start: Union[int, float, str, datetime.date] = None,
            end: Union[int, float, str, datetime.date] = None,
            columns: list = [],
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_all(**to_local(locals()))

    # 加载全部产品已有的K线
    def load_candle_map_all(
            self,
            instType: str,
            base_dir: str,
            symbols: list = [],
            endswith: str = '',
            contains: str = '',
            start: Union[int, float, str, datetime.date] = None,
            end: Union[int, float, str, datetime.date] = None,
            timezone: str = None,
            p_num: int = 1,
            columns: list = [],
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_map_all(**to_local(locals()))

    # 读取从start~end日期的历史K线数据
    def load_candle_by_date(
            self,
            instType: str,
            symbol: str,
            start: Union[int, float, str, datetime.date],
            end: Union[int, float, str, datetime.date],
            base_dir: str,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
            columns: list = [],
            valid_interval: bool = True,
            valid_start: bool = True,
            valid_end: bool = True,
    ) -> np.ndarray:
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_by_date(**to_local(locals()))

    # 按照日期读取candle_map
    def load_candle_map_by_date(
            self,
            instType: str,
            symbols: list,
            start: Union[int, float, str, datetime.date],
            end: Union[int, float, str, datetime.date],
            base_dir: str,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
            columns: list = [],
            endswith: str = '',
            contains: str = '',
            p_num: int = 1,
            valid_interval: bool = True,
            valid_start: bool = True,
            valid_end: bool = True,
    ) -> dict:
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_map_by_date(**to_local(locals()))

    # 通过文件地址读取Candle
    def load_candle_by_file(
            self,
            instType: str,
            symbol: str,
            columns: list = [],
            path: str = None,
            base_dir: str = '',
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
            valid_interval: bool = True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_by_file(**to_local(locals()))

    # 通过文件夹地址读取Candle_map
    def load_candle_map_by_file(
            self,
            instType: str,
            symbols: list = [],
            columns=[],
            path: str = None,
            base_dir: str = '',
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = '1m',
            valid_interval: bool = True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return load.load_candle_map_by_file(**to_local(locals()))

    # 按照日期保存Candle
    def save_candle_by_date(
            self,
            candle: np.array,
            instType: str,
            symbol: str,
            start: Union[int, float, str, datetime.date],
            end: Union[int, float, str, datetime.date],
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
            replace: bool = True,
            drop_duplicate: bool = True,
            sort: bool = True,
            valid_interval: bool = True,
            valid_start: bool = True,
            valid_end: bool = True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return save.save_candle_by_date(**to_local(locals()))

    # 按照日期保存candle_map
    def save_candle_map_by_date(
            self,
            candle_map: dict,
            instType: str,
            symbols: list,
            start: Union[int, float, str, datetime.date],
            end: Union[int, float, str, datetime.date],
            base_dir: str = '',
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
            drop_duplicate: bool = True,
            replace: bool = True,
            sort: bool = True,
            valid_interval: bool = True,
            valid_start: bool = True,
            valid_end: bool = True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return save.save_candle_map_by_date(**to_local(locals()))

    # 按照文件地址保存Candle
    def save_candle_by_file(
            self,
            candle: np.array,
            instType: str,
            symbol: str,
            path: str = None,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
            replace: bool = True,
            sort=True,
            drop_duplicate=True,
            valid_interval=True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return save.save_candle_by_file(**to_local(locals()))

    # 按照文件地址保存Candle_map
    def save_candle_map_by_file(
            self,
            candle_map: dict,
            instType: str,
            symbols: list,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
            replace: bool = True,
            sort: bool = True,
            drop_duplicate: bool = True,
            valid_interval: bool = True,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return save.save_candle_map_by_file(**to_local(locals()))

    # 获取某一个天candle的路径
    def get_candle_date_path(
            self,
            instType: str,
            symbol: str,
            date: datetime.date,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.get_candle_date_path(**to_local(locals()))

    # 获取candle文件的地址（一般不以天切割，必须缓存数据与1d数据可以储存在一个文件中）
    def get_candle_file_path(
            self,
            instType: str,
            symbol: str,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.get_candle_file_path(**to_local(locals()))

    # 检查candle从start到end日期数据文件是否齐全（仅检查文件是否存在，并不验证文件的准确性）
    def check_candle_date_path(
            self,
            instType: str,
            symbol: str,
            start: Union[int, float, str, datetime.date],
            end: Union[int, float, str, datetime.date],
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_DATE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.check_candle_date_path(**to_local(locals()))

    # 检查candle文件是否存在（不验证数据的准确性）
    def check_candle_file_path(
            self,
            instType: str,
            symbol: str,
            base_dir: str = None,
            timezone: str = None,
            bar: Literal['1m', '3m', '5m', '15m', '1H', '2H', '4H'] = None,
    ):
        if base_dir == None:
            base_dir = self.CANDLE_FILE_BASE_DIR
        if timezone == None:
            timezone = self.TIMEZONE
        if bar == None:
            bar = self.BAR
        return path.check_candle_file_path(**to_local(locals()))



from typing import Literal, Union
import numpy as np
from candlelite import exception

__all__ = ['history_suc', 'ma', 'boll', 'dualThrust']


# 按照开仓价格与平仓价格在历史数据中的成功次数
def history_suc(
        candle: np.array,
        posSide: Literal['long', 'short', 'LONG', 'SHORT'],
        buyLine: Union[int, float],
        sellLine: Union[int, float],
) -> int:
    '''
    :param candle: 历史K线数据
    :param posSide: 持仓方向
        long: 多单
        short: 空单
    :param buyLine: 开仓价格
    :param sellLine: 平仓价格
    :return: 历史成功次数
    '''
    this_order_index = 0
    suc_num = 0
    if posSide.upper() == 'LONG':
        while True:
            for index in range(this_order_index, candle.shape[0]):
                if candle[index, 3] <= buyLine:
                    this_order_index = index
                    break
                this_order_index = index
            for index in range(this_order_index + 1, candle.shape[0]):
                if candle[index, 2] >= sellLine:
                    this_order_index = index + 1
                    suc_num += 1
                    break
                this_order_index = index
            if this_order_index >= candle.shape[0] - 1:
                break
        return suc_num
    elif posSide.upper() == 'SHORT':
        while True:
            for index in range(this_order_index, candle.shape[0]):
                if candle[index, 2] >= buyLine:
                    this_order_index = index
                    break
                this_order_index = index
            for index in range(this_order_index + 1, candle.shape[0]):
                if candle[index, 3] <= sellLine:
                    this_order_index = index + 1
                    suc_num += 1
                    break
                this_order_index = index
            if this_order_index >= candle.shape[0] - 1:
                break
        return suc_num
    else:
        raise exception.PosSideException(
            func='history_suc',
            posSide=posSide,
        )


# Candle MA 均线
def ma(
        candle: np.array,
        n: int
) -> np.ndarray:
    '''
    :param candle:历史K线数据
    :param n:间隔
    :return:
        array([
                [ts,ma...],
                [ts.ma...],
                [ts.ma...],
            ])
    '''
    # list
    ma_datas = []
    candle_shape = candle.shape
    for index in range(candle_shape[0]):
        if index < n - 1:
            this_data = [candle[index, 0]] + [np.nan] * (candle_shape[1] - 1)
        else:
            this_data = [candle[index, 0]] + candle[index - n + 1:index + 1, 1:].mean(axis=0).tolist()
        ma_datas.append(this_data)
    # array
    candle_ma = np.array(ma_datas)
    return candle_ma


# Candle BOLL 布林带
def boll(
        candle: np.array,
        n: int
) -> np.ndarray:
    '''
    :param candle:历史K线数据
    :param n:间隔
    :return:
        array([
            [ts,mb,up,dn],
            [ts,mb,up,dn],
            [ts,mb,up,dn],
        ])
        (mb:均线 up:上轨 dn:下轨)
    '''
    # list
    boll_datas = []
    for index in range(candle.shape[0]):
        if index <= n - 1:
            mb = np.nan
            up = np.nan
            dn = np.nan
        else:
            mb = candle[index - n:index, 4].mean()  # 收盘价均值
            sd = (candle[index - n:index, 4] - mb).std()  # 收盘价标准差
            up = mb + 2 * sd  # 上轨
            dn = mb - 2 * sd  # 下轨
        boll_datas.append(
            [candle[index, 0], mb, up, dn]
        )
    # array
    candle_boll = np.array(boll_datas)
    return candle_boll


# 得到DualThrust中的上轨和下轨
def dualThrust(
        candle: np.array,
        n: int,
        ks: float,
        kx: float,
        update_n: int = None
) -> np.ndarray:
    '''
    :param candle:历史K线数据
    :param n:间隔
    :param ks:ks参数
    :param kx:kx参数
    :param update_n:更新的间隔n
    :return:
        array([
            [ts,range_ks,range_kx],
            [ts,range_ks,range_kx],
            [ts,range_ks,range_kx],
        ])
        (range_ks上轨 range_kx下轨)
    '''
    # list
    dualThrust_datas = []
    for i in range(0, candle.shape[0], update_n):
        if i <= n - 1:
            range_kx = np.nan
            range_ks = np.nan
        else:
            last_k_array = candle[i - n:i]  # 前n间隔的数据
            hh = last_k_array[:, 2].max()  # 最高价的最高价
            hc = last_k_array[:, 4].max()  # 最高价的收盘价
            lc = last_k_array[:, 4].min()  # 最低价的收盘价
            ll = last_k_array[:, 3].min()  # 最低价的最低价
            range_ = max(hh - lc, hc - ll)
            o = candle[i, 1]
            range_ks = o + ks * range_  # 上轨
            range_kx = o - kx * range_  # 下轨
        for ts in candle[i:i + update_n, 0]:
            dualThrust_datas.append(
                [ts, range_ks, range_kx]
            )
    # array
    candle_dualThrust = np.array(dualThrust_datas)
    return candle_dualThrust
